@@ -50,7 +50,7 @@ class CSVLoader:
             self, directory_root,
             conditions_directories=None, levels_labels=None,
             levels_inter=None, levels_intra=None, rows=None, columns=None,
-            inputs_axes_of_file_data=None, inputs_axis_of_samples=None,
+            csv_axes_inputs=None, batch_axis_inputs=None,
             batch_size=None, n_batches=None, shuffle=False, shifts=None,
             device=None, order_outputs='il'):
         """
@@ -222,9 +222,10 @@ class CSVLoader:
 
         self.indexes_samples = np.arange(self.n_samples)
 
-        indexes_samples_e, self.bin_indexes_samples = (
-            np.split(self.indexes_samples, [self.n_samples_e], axis=0))
-        if not self.shuffle:
+        indexes_samples_e, self.bin_indexes_samples = np.split(self.indexes_samples, [self.n_samples_e], axis=0)
+        self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+
+        if (not self.shuffle) and (self.size_bin_indexes_samples == 0):
             self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
 
         self.shifts = shifts  # type: Shifts
@@ -325,89 +326,82 @@ class CSVLoader:
 
         combination_directory_str_0 = [self.conditions_directories_names[l][0] for l in range(self.L)]
         directory_0 = os.path.join(self.directory_root, *combination_directory_str_0)
-        self.format_files = cp_directory.get_extension(directory_0, point=False).lower()
+        self.format_csv = cp_directory.get_extension(directory_0, point=False).lower()
 
         array_np_0 = cp_txt.csv_file_to_array(directory_0, rows=self.rows, columns=self.columns, dtype='f')
         tensor_0 = torch.tensor(array_np_0, dtype=torch.float32, device=self.device)
 
         shape_file_0 = list(tensor_0.shape)
-        self.shape_files = shape_file_0
-        self.n_dims_files = self.shape_files.__len__()
+        self.shape_csv = shape_file_0
+        self.n_dims_csv = self.shape_csv.__len__()
 
-        self.n_dims_samples = self.n_dims_files + self.H
+        self.n_dims_samples = self.n_dims_csv + self.H
         self.n_dims_batch = self.n_dims_samples + 1
 
         self.n_dims_directories_eb = self.H + 1
 
-        if inputs_axes_of_file_data is None:
-            self.inputs_axes_of_file_data = np.arange(
-                self.n_dims_batch - self.n_dims_files, self.n_dims_batch, 1, dtype='i')
+        if csv_axes_inputs is None:
+            self.csv_axes_inputs = np.arange(
+                self.n_dims_batch - self.n_dims_csv, self.n_dims_batch, 1, dtype='i')
         else:
             try:
-                len(inputs_axes_of_file_data)
-                if isinstance(inputs_axes_of_file_data, np.ndarray):
-                    self.inputs_axes_of_file_data = inputs_axes_of_file_data
+                len(csv_axes_inputs)
+                if isinstance(csv_axes_inputs, np.ndarray):
+                    self.csv_axes_inputs = csv_axes_inputs
                 else:
-                    self.inputs_axes_of_file_data = np.asarray(inputs_axes_of_file_data, dtype='i')
+                    self.csv_axes_inputs = np.asarray(csv_axes_inputs, dtype='i')
             except TypeError:
-                self.inputs_axes_of_file_data = np.asarray([inputs_axes_of_file_data], dtype='i')
+                self.csv_axes_inputs = np.asarray([csv_axes_inputs], dtype='i')
 
-        self.inputs_axes_of_file_data[self.inputs_axes_of_file_data < 0] += self.n_dims_batch
+        self.csv_axes_inputs[self.csv_axes_inputs < 0] += self.n_dims_batch
 
-        self.inputs_axes = np.arange(0, self.n_dims_batch, 1, dtype='i')
+        self.axes_inputs = np.arange(0, self.n_dims_batch, 1, dtype='i')
 
-        if inputs_axis_of_samples is None:
-            self.inputs_axis_of_samples = 0
-            while self.inputs_axis_of_samples in self.inputs_axes_of_file_data:
-                self.inputs_axis_of_samples += 1
-        elif inputs_axis_of_samples in self.inputs_axes_of_file_data:
-            if inputs_axes_of_file_data is None:
-                if inputs_axis_of_samples < 0:
-                    self.inputs_axis_of_samples = inputs_axis_of_samples + self.n_dims_batch
+        if batch_axis_inputs is None:
+            self.batch_axis_inputs = 0
+            while self.batch_axis_inputs in self.csv_axes_inputs:
+                self.batch_axis_inputs += 1
+        elif batch_axis_inputs in self.csv_axes_inputs:
+            if csv_axes_inputs is None:
+                if batch_axis_inputs < 0:
+                    self.batch_axis_inputs = batch_axis_inputs + self.n_dims_batch
                 else:
-                    self.inputs_axis_of_samples = inputs_axis_of_samples
-                self.inputs_axes_of_file_data[self.inputs_axes_of_file_data <= self.inputs_axis_of_samples] -= 1
+                    self.batch_axis_inputs = batch_axis_inputs
+                self.csv_axes_inputs[self.csv_axes_inputs <= self.batch_axis_inputs] -= 1
             else:
-                raise ValueError('inputs_axis_of_samples, inputs_axes_of_file_data')
+                raise ValueError('batch_axis_inputs, csv_axes_inputs')
         else:
-            self.inputs_axis_of_samples = inputs_axis_of_samples
+            self.batch_axis_inputs = batch_axis_inputs
 
-        if self.inputs_axis_of_samples < 0:
-            self.inputs_axis_of_samples += self.n_dims_batch
+        if self.batch_axis_inputs < 0:
+            self.batch_axis_inputs += self.n_dims_batch
 
-        inputs_axes_of_non_intra = np.sort(np.append(self.inputs_axis_of_samples, self.inputs_axes_of_file_data))
+        non_intra_axes_inputs = np.sort(np.append(self.batch_axis_inputs, self.csv_axes_inputs))
 
-        self.inputs_axes_of_intra = np.empty(self.H, dtype='i')
+        self.intra_axes_inputs = np.empty(self.H, dtype='i')
 
-        if len(self.inputs_axes_of_intra) > 0:
+        if len(self.intra_axes_inputs) > 0:
 
-            self.inputs_axes_of_intra[0] = 0
+            self.intra_axes_inputs[0] = 0
 
-            while self.inputs_axes_of_intra[0] in inputs_axes_of_non_intra:
-                self.inputs_axes_of_intra[0] += 1
+            while self.intra_axes_inputs[0] in non_intra_axes_inputs:
+                self.intra_axes_inputs[0] += 1
 
         for h in range(1, self.H):
-            self.inputs_axes_of_intra[h] = self.inputs_axes_of_intra[h - 1] + 1
-            while self.inputs_axes_of_intra[h] in inputs_axes_of_non_intra:
-                self.inputs_axes_of_intra[h] += 1
+            self.intra_axes_inputs[h] = self.intra_axes_inputs[h - 1] + 1
+            while self.intra_axes_inputs[h] in non_intra_axes_inputs:
+                self.intra_axes_inputs[h] += 1
 
-        self.input_axes_of_sample_data = self.inputs_axes[cp_array.samples_in_arr1_are_not_in_arr2(
-            self.inputs_axes, self.inputs_axis_of_samples)]
+        self.sample_axes_input = self.axes_inputs[cp_array.samples_in_arr1_are_not_in_arr2(
+            self.axes_inputs, self.batch_axis_inputs)]
 
         # self.batch_axes_of_intra = (
-        #     self.batch_axes[cp_array.samples_in_arr1_are_not_in_arr2(self.batch_axes, self.inputs_axes_of_file_data)])
+        #     self.batch_axes[cp_array.samples_in_arr1_are_not_in_arr2(self.batch_axes, self.csv_axes_inputs)])
 
         self.shape_batch = np.empty(self.n_dims_batch, dtype='i')
-        self.shape_batch[self.inputs_axis_of_samples] = self.batch_size
-        self.shape_batch[self.inputs_axes_of_intra] = self.n_conditions_directories_intra
-        self.shape_batch[self.inputs_axes_of_file_data] = self.shape_files
-
-        # self.shape_samples = tuple(self.shape_batch[self.input_axes_of_sample_data].tolist())
-        # self.shape_batch = tuple(self.shape_batch.tolist())
-        # self.shape_files = tuple(self.shape_files)
-
-        # self.shape_samples = tuple(np.append(self.n_conditions_directories_intra, self.shape_files, axis=0).tolist())
-        # self.shape_batch = tuple(np.append([self.batch_size], self.shape_samples, axis=0).tolist())
+        self.shape_batch[self.batch_axis_inputs] = self.batch_size
+        self.shape_batch[self.intra_axes_inputs] = self.n_conditions_directories_intra
+        self.shape_batch[self.csv_axes_inputs] = self.shape_csv
 
         if self.return_inputs_eb:
 
@@ -415,27 +409,27 @@ class CSVLoader:
         else:
             self.inputs_eb = None
 
-        self.directory_axes_of_intra = []
-        self.directory_axis_of_samples = None
+        self.intra_axes_directories = []
+        self.batch_axis_directories = None
 
         f = 0
-        for i in self.inputs_axes:
-            if i in self.inputs_axes_of_file_data:
+        for i in self.axes_inputs:
+            if i in self.csv_axes_inputs:
                 f += 1
-            elif i in self.inputs_axes_of_intra:
-                self.directory_axes_of_intra.append(i - f)
-            elif i == self.inputs_axis_of_samples:
-                self.directory_axis_of_samples = i - f
+            elif i in self.intra_axes_inputs:
+                self.intra_axes_directories.append(i - f)
+            elif i == self.batch_axis_inputs:
+                self.batch_axis_directories = i - f
 
         self.indexes_batch = np.empty(self.n_dims_batch, dtype='O')
         for d in range(0, self.n_dims_batch, 1):
             self.indexes_batch[d] = slice(0, self.shape_batch[d], 1)
 
-        self.directory_axes_of_intra = np.asarray(self.directory_axes_of_intra, dtype='i')
+        self.intra_axes_directories = np.asarray(self.intra_axes_directories, dtype='i')
 
         self.shape_directories_eb = np.empty(self.n_dims_directories_eb, dtype='i')
-        self.shape_directories_eb[self.directory_axes_of_intra] = self.shape_batch[self.inputs_axes_of_intra]
-        self.shape_directories_eb[self.directory_axis_of_samples] = self.shape_batch[self.inputs_axis_of_samples]
+        self.shape_directories_eb[self.intra_axes_directories] = self.shape_batch[self.intra_axes_inputs]
+        self.shape_directories_eb[self.batch_axis_directories] = self.shape_batch[self.batch_axis_inputs]
 
         if self.return_relative_directories_eb:
             self.relative_directories_eb = np.empty(self.shape_directories_eb, dtype='O')
@@ -495,12 +489,22 @@ class CSVLoader:
             self.shifts_intra.next()
 
         if self.shuffle:
-            indexes_samples_e = np.append(self.indexes_samples, self.bin_indexes_samples, axis=0)
-            indexes_samples_e, self.bin_indexes_samples = (
-                np.split(np.random.permutation(indexes_samples_e), [self.n_samples_e], axis=0))
-            self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
-            self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+            indexes_samples_e, self.bin_indexes_samples = np.split(
+                np.append(self.bin_indexes_samples, np.random.permutation(self.indexes_samples), axis=0),
+                [self.n_samples_e], axis=0)
 
+            self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+            self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
+        elif self.size_bin_indexes_samples > 0:
+
+            indexes_samples_e, self.bin_indexes_samples = np.split(
+                np.append(self.bin_indexes_samples, self.indexes_samples, axis=0),
+                [self.n_samples_e], axis=0)
+
+            self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+            self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
+
+        # for b in range(1):
         for b in range(self.n_batches):
             if self.return_labels_eb:
                 labels_eb = self.labels[self.batches_indexes[b]]
@@ -509,8 +513,8 @@ class CSVLoader:
             # batch_size_eb = batches_indexes_e.shape[0]
             for i in range(self.batch_size):
 
-                self.indexes_batch[self.inputs_axis_of_samples] = i
-                self.indexes_directories_ebij[self.directory_axis_of_samples] = i
+                self.indexes_batch[self.batch_axis_inputs] = i
+                self.indexes_directories_ebij[self.batch_axis_directories] = i
 
                 if self.shifts_intra is not None:
                     # TODO: recopy only the variable conditions with shifts
@@ -524,7 +528,6 @@ class CSVLoader:
 
                 if self.combinations_directories_intra.shape[0] > 0:
 
-
                     for j in range(self.combinations_directories_intra.shape[0]):
 
                         combination_intra_ebij = self.combinations_directories_intra[j, :]
@@ -538,18 +541,18 @@ class CSVLoader:
                         absolute_directory_ebij = os.path.join(self.directory_root, relative_directory_ebij)
 
                         if self.return_inputs_eb:
-                            self.indexes_batch[self.inputs_axes_of_intra] = self.combinations_indexes_input_intra[j, :]
+                            self.indexes_batch[self.intra_axes_inputs] = self.combinations_indexes_input_intra[j, :]
 
                             array_np_ebij = cp_txt.csv_file_to_array(
                                 absolute_directory_ebij, rows=self.rows, columns=self.columns, dtype='f')
                             tensor_ebij = torch.tensor(array_np_ebij, dtype=torch.float32, device=self.device)
 
                             # todo move dimensions of tensor_ebi
-                            #  if inputs_axes_of_file_data[0] > inputs_axes_of_file_data[1]
+                            #  if csv_axes_inputs[0] > csv_axes_inputs[1]
 
                             self.inputs_eb[tuple(self.indexes_batch)] = tensor_ebij
 
-                        self.indexes_directories_ebij[self.directory_axes_of_intra] = (
+                        self.indexes_directories_ebij[self.intra_axes_directories] = (
                             self.combinations_indexes_input_intra[j, :])
 
                         if self.return_relative_directories_eb:
@@ -572,7 +575,7 @@ class CSVLoader:
 
                         tensor_ebij = torch.tensor(array_np_ebij, dtype=torch.float32, device=self.device)
                         # todo move dimensions of tensor_ebi
-                        #  if inputs_axes_of_file_data[0] > inputs_axes_of_file_data[1]
+                        #  if csv_axes_inputs[0] > csv_axes_inputs[1]
 
                         self.inputs_eb[tuple(self.indexes_batch)] = tensor_ebij
 
