@@ -13,7 +13,6 @@ import math
 import os
 from .tools import Shifts
 from ... import array as cp_array
-from ... import txt as cp_txt
 from ... import directory as cp_directory
 from ... import maths as cp_maths
 from ... import combinations as cp_combinations
@@ -52,7 +51,7 @@ class ImageLoader:
             self, directory_root,
             conditions_directories=None, levels_labels=None,
             levels_inter=None, levels_intra=None,
-            inputs_axes_of_image=None, inputs_axis_of_samples=None,
+            image_axes_inputs=None, batch_axis_inputs=None,
             batch_size=None, n_batches=None, shuffle=False, shifts=None,
             transforms=None, device=None, order_outputs='il'):
         """
@@ -221,9 +220,10 @@ class ImageLoader:
 
         self.indexes_samples = np.arange(self.n_samples)
 
-        indexes_samples_e, self.bin_indexes_samples = (
-            np.split(self.indexes_samples, [self.n_samples_e], axis=0))
-        if not self.shuffle:
+        indexes_samples_e, self.bin_indexes_samples = np.split(self.indexes_samples, [self.n_samples_e], axis=0)
+        self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+
+        if (not self.shuffle) and (self.size_bin_indexes_samples == 0):
             self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
 
         self.shifts = shifts  # type: Shifts
@@ -329,91 +329,83 @@ class ImageLoader:
 
         combination_directory_str_0 = [self.conditions_directories_names[l][0] for l in range(self.L)]
         directory_0 = os.path.join(self.directory_root, *combination_directory_str_0)
-        self.format_images = cp_directory.get_extension(directory_0, point=False).lower()
+        self.format_image = cp_directory.get_extension(directory_0, point=False).lower()
 
         image_0 = PIL.Image.open(directory_0)
         tensor_0 = self.transforms(image_0)
 
 
         shape_image_0 = list(tensor_0.shape)
-        self.shape_images = shape_image_0
-        self.n_dims_images = self.shape_images.__len__()
+        self.shape_image = shape_image_0
+        self.n_dims_image = self.shape_image.__len__()
 
-        self.n_dims_samples = self.n_dims_images + self.H
+        self.n_dims_samples = self.n_dims_image + self.H
         self.n_dims_batch = self.n_dims_samples + 1
 
         self.n_dims_directories_eb = self.H + 1
 
-        if inputs_axes_of_image is None:
-            self.inputs_axes_of_image = np.arange(
-                self.n_dims_batch - self.n_dims_images, self.n_dims_batch, 1, dtype='i')
+        if image_axes_inputs is None:
+            self.image_axes_inputs = np.arange(
+                self.n_dims_batch - self.n_dims_image, self.n_dims_batch, 1, dtype='i')
         else:
             try:
-                len(inputs_axes_of_image)
-                if isinstance(inputs_axes_of_image, np.ndarray):
-                    self.inputs_axes_of_image = inputs_axes_of_image
+                len(image_axes_inputs)
+                if isinstance(image_axes_inputs, np.ndarray):
+                    self.image_axes_inputs = image_axes_inputs
                 else:
-                    self.inputs_axes_of_image = np.asarray(inputs_axes_of_image, dtype='i')
+                    self.image_axes_inputs = np.asarray(image_axes_inputs, dtype='i')
             except TypeError:
-                self.inputs_axes_of_image = np.asarray([inputs_axes_of_image], dtype='i')
+                self.image_axes_inputs = np.asarray([image_axes_inputs], dtype='i')
 
-        self.inputs_axes_of_image[self.inputs_axes_of_image < 0] += self.n_dims_batch
+        self.image_axes_inputs[self.image_axes_inputs < 0] += self.n_dims_batch
 
-        self.inputs_axes = np.arange(0, self.n_dims_batch, 1, dtype='i')
+        self.axes_inputs = np.arange(0, self.n_dims_batch, 1, dtype='i')
 
-        if inputs_axis_of_samples is None:
-            self.inputs_axis_of_samples = 0
-            while self.inputs_axis_of_samples in self.inputs_axes_of_image:
-                self.inputs_axis_of_samples += 1
-        elif inputs_axis_of_samples in self.inputs_axes_of_image:
-            if inputs_axes_of_image is None:
-                if inputs_axis_of_samples < 0:
-                    self.inputs_axis_of_samples = inputs_axis_of_samples + self.n_dims_batch
+        if batch_axis_inputs is None:
+            self.batch_axis_inputs = 0
+            while self.batch_axis_inputs in self.image_axes_inputs:
+                self.batch_axis_inputs += 1
+        elif batch_axis_inputs in self.image_axes_inputs:
+            if image_axes_inputs is None:
+                if batch_axis_inputs < 0:
+                    self.batch_axis_inputs = batch_axis_inputs + self.n_dims_batch
                 else:
-                    self.inputs_axis_of_samples = inputs_axis_of_samples
-                self.inputs_axes_of_image[self.inputs_axes_of_image <= self.inputs_axis_of_samples] -= 1
+                    self.batch_axis_inputs = batch_axis_inputs
+                self.image_axes_inputs[self.image_axes_inputs <= self.batch_axis_inputs] -= 1
             else:
-                raise ValueError('inputs_axis_of_samples, inputs_axes_of_image')
+                raise ValueError('batch_axis_inputs, image_axes_inputs')
         else:
-            self.inputs_axis_of_samples = inputs_axis_of_samples
+            self.batch_axis_inputs = batch_axis_inputs
 
-        if self.inputs_axis_of_samples < 0:
-            self.inputs_axis_of_samples += self.n_dims_batch
+        if self.batch_axis_inputs < 0:
+            self.batch_axis_inputs += self.n_dims_batch
 
-        inputs_axes_of_non_intra = np.sort(np.append(self.inputs_axis_of_samples, self.inputs_axes_of_image))
+        non_intra_axes_inputs = np.sort(np.append(self.batch_axis_inputs, self.image_axes_inputs))
 
-        self.inputs_axes_of_intra = np.empty(self.H, dtype='i')
+        self.intra_axes_inputs = np.empty(self.H, dtype='i')
 
-        if len(self.inputs_axes_of_intra) > 0:
+        if len(self.intra_axes_inputs) > 0:
 
-            self.inputs_axes_of_intra[0] = 0
+            self.intra_axes_inputs[0] = 0
 
-            while self.inputs_axes_of_intra[0] in inputs_axes_of_non_intra:
-                self.inputs_axes_of_intra[0] += 1
+            while self.intra_axes_inputs[0] in non_intra_axes_inputs:
+                self.intra_axes_inputs[0] += 1
 
         for h in range(1, self.H):
-            self.inputs_axes_of_intra[h] = self.inputs_axes_of_intra[h - 1] + 1
-            while self.inputs_axes_of_intra[h] in inputs_axes_of_non_intra:
-                self.inputs_axes_of_intra[h] += 1
+            self.intra_axes_inputs[h] = self.intra_axes_inputs[h - 1] + 1
+            while self.intra_axes_inputs[h] in non_intra_axes_inputs:
+                self.intra_axes_inputs[h] += 1
 
-        self.input_axes_of_sample_data = self.inputs_axes[cp_array.samples_in_arr1_are_not_in_arr2(
-            self.inputs_axes, self.inputs_axis_of_samples)]
+        self.sample_axes_input = self.axes_inputs[cp_array.samples_in_arr1_are_not_in_arr2(
+            self.axes_inputs, self.batch_axis_inputs)]
 
         # self.batch_axes_of_intra = (
-        #     self.batch_axes[cp_array.samples_in_arr1_are_not_in_arr2(
-        #         self.batch_axes, self.inputs_axes_of_image)])
+        #     self.batch_axes[cp_array.samples_in_arr1_are_not_in_arr2(self.batch_axes, self.image_axes_inputs)])
 
         self.shape_batch = np.empty(self.n_dims_batch, dtype='i')
-        self.shape_batch[self.inputs_axis_of_samples] = self.batch_size
-        self.shape_batch[self.inputs_axes_of_intra] = self.n_conditions_directories_intra
-        self.shape_batch[self.inputs_axes_of_image] = self.shape_images
-
-        # self.shape_samples = tuple(self.shape_batch[self.input_axes_of_sample_data].tolist())
-        # self.shape_batch = tuple(self.shape_batch.tolist())
-        # self.shape_images = tuple(self.shape_images)
-
-        # self.shape_samples = tuple(np.append(self.n_conditions_directories_intra, self.shape_images, axis=0).tolist())
-        # self.shape_batch = tuple(np.append([self.batch_size], self.shape_samples, axis=0).tolist())
+        self.shape_batch[self.batch_axis_inputs] = self.batch_size
+        self.shape_batch[self.intra_axes_inputs] = self.n_conditions_directories_intra
+        self.shape_batch[self.image_axes_inputs] = self.shape_image
 
         if self.return_inputs_eb:
 
@@ -421,27 +413,27 @@ class ImageLoader:
         else:
             self.inputs_eb = None
 
-        self.directory_axes_of_intra = []
-        self.directory_axis_of_samples = None
+        self.intra_axes_directories = []
+        self.batch_axis_directories = None
 
         f = 0
-        for i in self.inputs_axes:
-            if i in self.inputs_axes_of_image:
+        for i in self.axes_inputs:
+            if i in self.image_axes_inputs:
                 f += 1
-            elif i in self.inputs_axes_of_intra:
-                self.directory_axes_of_intra.append(i - f)
-            elif i == self.inputs_axis_of_samples:
-                self.directory_axis_of_samples = i - f
+            elif i in self.intra_axes_inputs:
+                self.intra_axes_directories.append(i - f)
+            elif i == self.batch_axis_inputs:
+                self.batch_axis_directories = i - f
 
         self.indexes_batch = np.empty(self.n_dims_batch, dtype='O')
         for d in range(0, self.n_dims_batch, 1):
             self.indexes_batch[d] = slice(0, self.shape_batch[d], 1)
 
-        self.directory_axes_of_intra = np.asarray(self.directory_axes_of_intra, dtype='i')
+        self.intra_axes_directories = np.asarray(self.intra_axes_directories, dtype='i')
 
         self.shape_directories_eb = np.empty(self.n_dims_directories_eb, dtype='i')
-        self.shape_directories_eb[self.directory_axes_of_intra] = self.shape_batch[self.inputs_axes_of_intra]
-        self.shape_directories_eb[self.directory_axis_of_samples] = self.shape_batch[self.inputs_axis_of_samples]
+        self.shape_directories_eb[self.intra_axes_directories] = self.shape_batch[self.intra_axes_inputs]
+        self.shape_directories_eb[self.batch_axis_directories] = self.shape_batch[self.batch_axis_inputs]
 
         if self.return_relative_directories_eb:
             self.relative_directories_eb = np.empty(self.shape_directories_eb, dtype='O')
@@ -501,12 +493,22 @@ class ImageLoader:
             self.shifts_intra.next()
 
         if self.shuffle:
-            indexes_samples_e = np.append(self.indexes_samples, self.bin_indexes_samples, axis=0)
-            indexes_samples_e, self.bin_indexes_samples = (
-                np.split(np.random.permutation(indexes_samples_e), [self.n_samples_e], axis=0))
-            self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
-            self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+            indexes_samples_e, self.bin_indexes_samples = np.split(
+                np.append(self.bin_indexes_samples, np.random.permutation(self.indexes_samples), axis=0),
+                [self.n_samples_e], axis=0)
 
+            self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+            self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
+        elif self.size_bin_indexes_samples > 0:
+
+            indexes_samples_e, self.bin_indexes_samples = np.split(
+                np.append(self.bin_indexes_samples, self.indexes_samples, axis=0),
+                [self.n_samples_e], axis=0)
+
+            self.size_bin_indexes_samples = len(self.bin_indexes_samples)
+            self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
+
+        # for b in range(1):
         for b in range(self.n_batches):
             if self.return_labels_eb:
                 labels_eb = self.labels[self.batches_indexes[b]]
@@ -515,9 +517,8 @@ class ImageLoader:
             # batch_size_eb = batches_indexes_e.shape[0]
             for i in range(self.batch_size):
 
-                self.indexes_batch[self.inputs_axis_of_samples] = i
-                self.indexes_directories_ebij[self.directory_axis_of_samples] = i
-
+                self.indexes_batch[self.batch_axis_inputs] = i
+                self.indexes_directories_ebij[self.batch_axis_directories] = i
 
                 if self.shifts_intra is not None:
                     # TODO: recopy only the variable conditions with shifts
@@ -531,7 +532,6 @@ class ImageLoader:
 
                 if self.combinations_directories_intra.shape[0] > 0:
 
-
                     for j in range(self.combinations_directories_intra.shape[0]):
 
                         combination_intra_ebij = self.combinations_directories_intra[j, :]
@@ -545,17 +545,17 @@ class ImageLoader:
                         absolute_directory_ebij = os.path.join(self.directory_root, relative_directory_ebij)
 
                         if self.return_inputs_eb:
-                            self.indexes_batch[self.inputs_axes_of_intra] = self.combinations_indexes_input_intra[j, :]
+                            self.indexes_batch[self.intra_axes_inputs] = self.combinations_indexes_input_intra[j, :]
 
                             image_ebij = PIL.Image.open(absolute_directory_ebij)
                             tensor_ebij = self.transforms(image_ebij)
 
                             # todo move dimensions of tensor_ebi
-                            #  if inputs_axes_of_image[0] > inputs_axes_of_image[1]
+                            #  if image_axes_inputs[0] > image_axes_inputs[1]
 
                             self.inputs_eb[tuple(self.indexes_batch)] = tensor_ebij
 
-                        self.indexes_directories_ebij[self.directory_axes_of_intra] = (
+                        self.indexes_directories_ebij[self.intra_axes_directories] = (
                             self.combinations_indexes_input_intra[j, :])
 
                         if self.return_relative_directories_eb:
@@ -577,7 +577,7 @@ class ImageLoader:
                         tensor_ebij = self.transforms(image_ebij)
 
                         # todo move dimensions of tensor_ebi
-                        #  if inputs_axes_of_image[0] > inputs_axes_of_image[1]
+                        #  if image_axes_inputs[0] > image_axes_inputs[1]
 
                         self.inputs_eb[tuple(self.indexes_batch)] = tensor_ebij
 
